/*
 * Copyright 2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */
package com.sunlabs.archsim.fleet.ships.memory;

import com.sunlabs.archsim.base.common.Parameters;
import com.sunlabs.archsim.base.simulation.Command;
import com.sunlabs.archsim.base.simulation.Delay;
import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.fleet.ships.testing.FunctionStep;
import com.sunlabs.archsim.fleet.ships.testing.ScriptedFunction;
import com.sunlabs.archsim.gasp.Component;
import com.sunlabs.archsim.gasp.Function;
import com.sunlabs.archsim.gasp.Terminal;
import com.sunlabs.archsim.gasp.TriggerAction;
import com.sunlabs.archsim.gasp.terminals.InputTerminal;
import com.sunlabs.archsim.gasp.terminals.OutputTerminal;

public class SimpleMemoryTestSource extends Component {

	static final Delay readSourceDelay= new Delay(10);
	static final Delay writeSourceDelay= new Delay(10);
	
	
	SimpleMemoryReadSource readSource= new SimpleMemoryReadSource();
	SimpleMemoryWriteSource writeSource= new SimpleMemoryWriteSource();
	
	Command[] stopReadSource= { 
			readSource.disableTriggerCommand 
	};
	
	Command[] stopReadStartWriteSource= {
		readSource.disableTriggerCommand, 
		writeSource.enableTriggerCommand
	};
	
	Command[] stopWriteStartReadSource= {
			readSource.enableTriggerCommand, 
			writeSource.disableTriggerCommand			
	};
	
	FunctionStep[] readSteps = {
		new FunctionStep(new Data(0)),
		new FunctionStep(new Data(MemoryDefaults.MEMORY_SIZE_DEF-1)),
		new FunctionStep(new Data(100), stopReadStartWriteSource ),
		new FunctionStep(new Data(100)),
		new FunctionStep(new Data(MemoryDefaults.MEMORY_SIZE_DEF-1)),
		new FunctionStep(new Data(0), stopReadSource)		
	};
	
	FunctionStep[] writeAddressSteps= {
		new FunctionStep(new Data(100)),
		new FunctionStep(new Data(MemoryDefaults.MEMORY_SIZE_DEF-1)),
		new FunctionStep(new Data(0), stopWriteStartReadSource)
	};
	
	FunctionStep[] writeDataSteps= {
		new FunctionStep(new Data(1)),
		new FunctionStep(new Data(2)),
		new FunctionStep(new Data(3)),		
	};
		
	
	Terminal readAddressTerm= new OutputTerminal(MemoryDefaults.READ_ADDRESS, this);
	Terminal ackReadAddressTerm= new InputTerminal(MemoryDefaults.ACK_READ_ADDRESS, this);
	
	Terminal writeAddressTerm= new OutputTerminal(MemoryDefaults.WRITE_ADDRESS, this);
	Terminal ackWriteAddressTerm= new InputTerminal(MemoryDefaults.ACK_WRITE_ADDRESS, this);
	
	Terminal writeDataTerm= new OutputTerminal(MemoryDefaults.WRITE_DATA, this);
	Terminal ackWriteDataTerm= new InputTerminal(MemoryDefaults.ACK_WRITE_DATA, this);
	
	public SimpleMemoryTestSource(String name, Parameters params) {
		super(name, null);
		
		

		addMember(readSource);
		addMember(writeSource);
		
		// set steps for the sources
		readSource.setSteps();
		writeSource.setSteps();
		
		// attach terminals
		readSource.attach(readAddressTerm);
		readSource.attach(ackReadAddressTerm);
		
		writeSource.attach(writeAddressTerm);
		writeSource.attach(ackWriteAddressTerm);
		writeSource.attach(writeDataTerm);
		writeSource.attach(ackWriteDataTerm);
		
		// enable read source
		readSource.enableTriggerCommand.execute(null);
			
	} // constructor SimpleMemoryTestSource
	
	
	public Terminal getTerminal(String name) {
		if (name.equals(MemoryDefaults.READ_ADDRESS)) {
			return readAddressTerm;
		}
		else if (name.equals(MemoryDefaults.ACK_READ_ADDRESS)) {
			return ackReadAddressTerm;
		}
		else if (name.equals(MemoryDefaults.WRITE_ADDRESS)) {
			return writeAddressTerm;
		}
		else if (name.equals(MemoryDefaults.ACK_WRITE_ADDRESS)) {
			return ackWriteAddressTerm;
		}
		else if (name.equals(MemoryDefaults.WRITE_DATA)) {
			return writeDataTerm;
		}
		else if (name.equals(MemoryDefaults.ACK_WRITE_DATA)) {
			return ackWriteDataTerm;
		}
		else {
			return null;
		}
	} // getTerminal
	
	
	class SimpleMemoryReadSource extends TriggerAction {
		
		Function generateAddressFn;
		
		public SimpleMemoryReadSource() {
			super("SimpleMemoryReadSource");
		}
		
		void setSteps() {
			generateAddressFn= new ScriptedFunction("SimpleMemoryReadTest", readSteps);
			generateAddressFn.setDelay(readSourceDelay);
			outputFunctions.put(MemoryDefaults.READ_ADDRESS, generateAddressFn);			
		} // setSteps
		
	} // class SimpleMemoryReadSource
	
	
	class SimpleMemoryWriteSource extends TriggerAction {
		
		Function generateAddressFn;
		Function generateDataFn;
		
		public SimpleMemoryWriteSource() {
			super("SimpleMemoryWriteSource");
		}
		
		void setSteps() {
			generateAddressFn= new ScriptedFunction("SimpleMemoryWriteTest", writeAddressSteps);
			generateAddressFn.setDelay(writeSourceDelay);
			outputFunctions.put(MemoryDefaults.WRITE_ADDRESS, generateAddressFn);			

			generateDataFn= new ScriptedFunction("SimpleMemoryWriteTest", writeDataSteps);
			generateDataFn.setDelay(writeSourceDelay);
			outputFunctions.put(MemoryDefaults.WRITE_DATA, generateDataFn);			
		} // setSteps
	
	} // class SimpleMemoryWriteSource
		
} // class SimpleMemoryTestSource

